@@ -11,6 +11,7 @@
 #include <errno.h>
 #include <assert.h>
 #include <sys/poll.h>
+#include <sys/mman.h>
 
 #include "../fio.h"
 
@@ -19,6 +20,7 @@
 struct spliceio_data {
 	int pipe[2];
 	int vmsplice_to_user;
+	int vmsplice_to_user_map;
 };
 
 /*
@@ -75,25 +77,44 @@ static int fio_splice_read(struct thread_data *td, struct io_u *io_u)
 	struct spliceio_data *sd = td->io_ops->data;
 	struct fio_file *f = io_u->file;
 	struct iovec iov;
-	int ret, buflen;
+	int ret , buflen, mmap_len;
 	off_t offset;
-	void *p;
+	void *p, *map;
 
+restart:
+	ret = 0;
 	offset = io_u->offset;
-	buflen = io_u->xfer_buflen;
-	p = io_u->xfer_buf;
+	mmap_len = buflen = io_u->xfer_buflen;
+
+	if (sd->vmsplice_to_user_map) {
+		map = mmap(io_u->xfer_buf, buflen, PROT_READ, MAP_PRIVATE|OS_MAP_ANON, 0, 0);
+		if (map == MAP_FAILED) {
+			td_verror(td, errno, "mmap io_u");
+			return -1;
+		}
+
+		p = map;
+	} else {
+		map = NULL;
+		p = io_u->xfer_buf;
+	}
+
 	while (buflen) {
 		int this_len = buflen;
+		int flags = 0;
 
-		if (this_len > SPLICE_DEF_SIZE)
+		if (this_len > SPLICE_DEF_SIZE) {
 			this_len = SPLICE_DEF_SIZE;
+			flags = SPLICE_F_MORE;
+		}
 
-		ret = splice(f->fd, &offset, sd->pipe[1], NULL, this_len, SPLICE_F_MORE);
+		ret = splice(f->fd, &offset, sd->pipe[1], NULL, this_len,flags);
 		if (ret < 0) {
 			if (errno == ENODATA || errno == EAGAIN)
 				continue;
 
-			return -errno;
+			td_verror(td, errno, "splice-from-fd");
+			break;
 		}
 
 		buflen -= ret;
@@ -103,16 +124,38 @@ static int fio_splice_read(struct thread_data *td, struct io_u *io_u)
 
 		while (iov.iov_len) {
 			ret = vmsplice(sd->pipe[0], &iov, 1, SPLICE_F_MOVE);
-			if (ret < 0)
-				return -errno;
-			else if (!ret)
-				return -ENODATA;
+			if (ret < 0) {
+				if (errno == EFAULT && sd->vmsplice_to_user_map) {
+					sd->vmsplice_to_user_map = 0;
+					munmap(map, mmap_len);
+					goto restart;
+				}
+				if (errno == EBADF) {
+					ret = -EBADF;
+					break;
+				}
+				td_verror(td, errno, "vmsplice");
+				break;
+			} else if (!ret) {
+				td_verror(td, ENODATA, "vmsplice");
+				ret = -1;
+				break;
+			}
 
 			iov.iov_len -= ret;
 			iov.iov_base += ret;
 		}
+		if (ret < 0)
+			break;
 	}
 
+	if (sd->vmsplice_to_user_map && munmap(map, mmap_len) < 0) {
+		td_verror(td, errno, "munnap io_u");
+		return -1;
+	}
+	if (ret < 0)
+		return ret;
+
 	return io_u->xfer_buflen;
 }
 
@@ -160,6 +203,8 @@ static int fio_spliceio_queue(struct thread_data *td, struct io_u *io_u)
 	struct spliceio_data *sd = td->io_ops->data;
 	int ret;
 
+	fio_ro_check(td, io_u);
+
 	if (io_u->ddir == DDIR_READ) {
 		if (sd->vmsplice_to_user) {
 			ret = fio_splice_read(td, io_u);
@@ -220,6 +265,19 @@ static int fio_spliceio_init(struct thread_data *td)
 	 */
 	sd->vmsplice_to_user = 1;
 
+	/*
+	 * Works with "real" vmsplice to user, eg mapping pages directly.
+	 * Reset if we fail.
+	 */
+	sd->vmsplice_to_user_map = 1;
+
+	/*
+	 * And if vmsplice_to_user works, we definitely need aligned
+	 * buffers. Just set ->odirect to force that.
+	 */
+	if (td_read(td))
+		td->o.odirect = 1;
+
 	td->io_ops->data = sd;
 	return 0;
 }
