@@ -13,7 +13,6 @@
 #include <asm/unistd.h>
 
 #include "../fio.h"
-#include "../os.h"
 
 #ifdef FIO_HAVE_SYSLET
 
@@ -25,6 +24,8 @@
 #define __NR_fio_pwrite	__NR_pwrite
 #endif
 
+#define ATOM_TO_IOU(p)	((struct io_u *) (unsigned long) (p))
+
 struct syslet_data {
 	struct io_u **events;
 	unsigned int nr_events;
@@ -47,7 +48,7 @@ static void fio_syslet_complete_atom(struct thread_data *td,
 	 * including) this atom
 	 */
 	last = atom;
-	io_u = atom->private;
+	io_u = ATOM_TO_IOU(atom);
 	atom = io_u->req.head;
 
 	/*
@@ -56,8 +57,8 @@ static void fio_syslet_complete_atom(struct thread_data *td,
 	do {
 		long ret;
 
-		io_u = atom->private;
-		ret = *atom->ret_ptr;
+		io_u = ATOM_TO_IOU(atom);
+		ret = *(long *) (unsigned long) atom->ret_ptr;
 		if (ret >= 0)
 			io_u->resid = io_u->xfer_buflen - ret;
 		else if (ret < 0)
@@ -69,7 +70,7 @@ static void fio_syslet_complete_atom(struct thread_data *td,
 		if (atom == last)
 			break;
 
-		atom = atom->next;
+		atom = (struct syslet_uatom *) (unsigned long) atom->next;
 	} while (1);
 
 	assert(!last->next);
@@ -139,14 +140,15 @@ static void init_atom(struct syslet_uatom *atom, int nr, void *arg0,
 {
 	atom->flags = flags;
 	atom->nr = nr;
-	atom->ret_ptr = ret_ptr;
-	atom->next = NULL;
-	atom->arg_ptr[0] = arg0;
-	atom->arg_ptr[1] = arg1;
-	atom->arg_ptr[2] = arg2;
-	atom->arg_ptr[3] = arg3;
-	atom->arg_ptr[4] = atom->arg_ptr[5] = NULL;
-	atom->private = priv;
+	atom->ret_ptr = (uint64_t) (unsigned long) ret_ptr;
+	atom->next = 0;
+	atom->arg_ptr[0] = (uint64_t) (unsigned long) arg0;
+	atom->arg_ptr[1] = (uint64_t) (unsigned long) arg1;
+	atom->arg_ptr[2] = (uint64_t) (unsigned long) arg2;
+	atom->arg_ptr[3] = (uint64_t) (unsigned long) arg3;
+	atom->arg_ptr[4] = 0;
+	atom->arg_ptr[5] = 0;
+	atom->private = (uint64_t) (unsigned long) priv;
 }
 
 /*
@@ -208,11 +210,11 @@ static void fio_syslet_queued(struct thread_data *td, struct syslet_data *sd)
 
 	atom = sd->head;
 	while (atom) {
-		struct io_u *io_u = atom->private;
+		struct io_u *io_u = ATOM_TO_IOU(atom);
 
 		memcpy(&io_u->issue_time, &now, sizeof(now));
 		io_u_queued(td, io_u);
-		atom = atom->next;
+		atom = (struct syslet_uatom *) (unsigned long) atom->next;
 	}
 }
 
@@ -254,11 +256,13 @@ static int fio_syslet_queue(struct thread_data *td, struct io_u *io_u)
 {
 	struct syslet_data *sd = td->io_ops->data;
 
+	fio_ro_check(td, io_u);
+
 	if (sd->tail) {
-		sd->tail->next = &io_u->req.atom;
+		sd->tail->next = (uint64_t) (unsigned long) &io_u->req.atom;
 		sd->tail = &io_u->req.atom;
 	} else
-		sd->head = sd->tail = &io_u->req.atom;
+		sd->head = sd->tail = (struct syslet_uatom *)&io_u->req.atom;
 
 	io_u->req.head = sd->head;
 	return FIO_Q_QUEUED;
@@ -275,11 +279,12 @@ static int async_head_init(struct syslet_data *sd, unsigned int depth)
 	memset(sd->ring, 0, ring_size);
 
 	sd->ahu.user_ring_idx = 0;
-	sd->ahu.completion_ring = sd->ring;
+	sd->ahu.completion_ring_ptr = (uint64_t) (unsigned long) sd->ring;
 	sd->ahu.ring_size_bytes = ring_size;
 	sd->ahu.head_stack = thread_stack_alloc();
-	sd->ahu.head_eip = (unsigned long) cachemiss_thread_start;
-	sd->ahu.new_thread_eip = (unsigned long) cachemiss_thread_start;
+	sd->ahu.head_ip = (uint64_t) (unsigned long) cachemiss_thread_start;
+	sd->ahu.new_thread_ip = (uint64_t) (unsigned long) cachemiss_thread_start;
+	sd->ahu.new_thread_stack = thread_stack_alloc();
 
 	return 0;
 }
@@ -295,7 +300,7 @@ static int check_syslet_support(struct syslet_data *sd)
 	void *ret;
 
 	init_atom(&atom, __NR_getpid, NULL, NULL, NULL, NULL, NULL, 0, NULL);
-	ret = async_exec(sd->head, &sd->ahu);
+	ret = async_exec(&atom, &sd->ahu);
 	if (ret == (void *) -1)
 		return 1;
 
