@@ -18,6 +18,11 @@ static int extend_file(struct thread_data *td, struct fio_file *f)
 	unsigned int bs;
 	char *b;
 
+	if (read_only) {
+		log_err("fio: refusing extend of file due to read-only\n");
+		return 0;
+	}
+
 	/*
 	 * check if we need to lay the file out complete again. fio
 	 * does that for operations involving reads, or for writes
@@ -226,8 +231,12 @@ int generic_open_file(struct thread_data *td, struct fio_file *f)
 		flags |= OS_O_DIRECT;
 	if (td->o.sync_io)
 		flags |= O_SYNC;
+	if (f->filetype != FIO_TYPE_FILE)
+		flags |= O_NOATIME;
 
 	if (td_write(td)) {
+		assert(!read_only);
+
 		flags |= O_RDWR;
 
 		if (f->filetype == FIO_TYPE_FILE)
@@ -238,7 +247,7 @@ int generic_open_file(struct thread_data *td, struct fio_file *f)
 		else
 			f->fd = open(f->file_name, flags, 0600);
 	} else {
-		if (f->filetype == FIO_TYPE_CHAR)
+		if (f->filetype == FIO_TYPE_CHAR && !read_only)
 			flags |= O_RDWR;
 		else
 			flags |= O_RDONLY;
@@ -364,7 +373,8 @@ int setup_files(struct thread_data *td)
 	/*
 	 * device/file sizes are zero and no size given, punt
 	 */
-	if ((!total_size || total_size == -1ULL) && !td->o.size) {
+	if ((!total_size || total_size == -1ULL) && !td->o.size &&
+	    !(td->io_ops->flags & FIO_NOIO)) {
 		log_err("%s: you need to specify size=\n", td->o.name);
 		td_verror(td, EINVAL, "total_file_size");
 		return 1;
